# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.ach_holder_type import AchHolderType
from ...types.ach_sec_code import AchSecCode
from ...types.batch_number import BatchNumber
from ...types.chargeback_id import ChargebackId
from ...types.dbaname import Dbaname
from ...types.device import Device
from ...types.entrypage_id import EntrypageId
from ...types.entrypointfield import Entrypointfield
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.external_processor_information import ExternalProcessorInformation
from ...types.fee_amount import FeeAmount
from ...types.ip_address import IpAddress
from ...types.legalname import Legalname
from ...types.operation import Operation
from ...types.order_id import OrderId
from ...types.org_parent_name import OrgParentName
from ...types.orgid import Orgid
from ...types.payor_id import PayorId
from ...types.paypoint_id import PaypointId
from ...types.pending_fee_amount import PendingFeeAmount
from ...types.query_c_fee_transaction import QueryCFeeTransaction
from ...types.refund_id import RefundId
from ...types.retrieval_id import RetrievalId
from ...types.returned_id import ReturnedId
from ...types.risk_action import RiskAction
from ...types.risk_action_code import RiskActionCode
from ...types.risk_flagged import RiskFlagged
from ...types.risk_flagged_on import RiskFlaggedOn
from ...types.risk_reason import RiskReason
from ...types.risk_status import RiskStatus
from ...types.settlement_status import SettlementStatus
from ...types.source import Source
from ...types.split_funding import SplitFunding
from ...types.trans_status import TransStatus
from .transaction_detail_customer import TransactionDetailCustomer
from .transaction_detail_event import TransactionDetailEvent
from .transaction_detail_invoice_data import TransactionDetailInvoiceData
from .transaction_detail_payment_data import TransactionDetailPaymentData
from .transaction_detail_record_method import TransactionDetailRecordMethod
from .transaction_detail_response_data import TransactionDetailResponseData


class TransactionDetailRecord(UniversalBaseModel):
    """
    Complete transaction details including payment information, customer data, and processing metadata. This is returned when includeDetails=true.
    """

    parent_org_name: typing_extensions.Annotated[
        OrgParentName, FieldMetadata(alias="parentOrgName"), pydantic.Field(alias="parentOrgName")
    ]
    paypoint_dbaname: typing_extensions.Annotated[
        Dbaname, FieldMetadata(alias="paypointDbaname"), pydantic.Field(alias="paypointDbaname")
    ]
    paypoint_legalname: typing_extensions.Annotated[
        Legalname, FieldMetadata(alias="paypointLegalname"), pydantic.Field(alias="paypointLegalname")
    ]
    paypoint_entryname: typing_extensions.Annotated[
        Entrypointfield, FieldMetadata(alias="paypointEntryname"), pydantic.Field(alias="paypointEntryname")
    ]
    payment_trans_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="paymentTransId"), pydantic.Field(alias="paymentTransId")
    ]
    connector_name: typing_extensions.Annotated[
        str, FieldMetadata(alias="connectorName"), pydantic.Field(alias="connectorName")
    ]
    external_processor_information: typing_extensions.Annotated[
        ExternalProcessorInformation,
        FieldMetadata(alias="externalProcessorInformation"),
        pydantic.Field(alias="externalProcessorInformation"),
    ]
    gateway_trans_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="gatewayTransId"), pydantic.Field(alias="gatewayTransId")
    ]
    order_id: typing_extensions.Annotated[
        typing.Optional[OrderId], FieldMetadata(alias="orderId"), pydantic.Field(alias="orderId")
    ] = None
    method: TransactionDetailRecordMethod
    batch_number: typing_extensions.Annotated[
        BatchNumber, FieldMetadata(alias="batchNumber"), pydantic.Field(alias="batchNumber")
    ]
    batch_amount: typing_extensions.Annotated[
        float, FieldMetadata(alias="batchAmount"), pydantic.Field(alias="batchAmount")
    ]
    payor_id: typing_extensions.Annotated[PayorId, FieldMetadata(alias="payorId"), pydantic.Field(alias="payorId")]
    payment_data: typing_extensions.Annotated[
        TransactionDetailPaymentData, FieldMetadata(alias="paymentData"), pydantic.Field(alias="paymentData")
    ]
    trans_status: typing_extensions.Annotated[
        TransStatus, FieldMetadata(alias="transStatus"), pydantic.Field(alias="transStatus")
    ]
    paypoint_id: typing_extensions.Annotated[
        PaypointId, FieldMetadata(alias="paypointId"), pydantic.Field(alias="paypointId")
    ]
    total_amount: typing_extensions.Annotated[
        float, FieldMetadata(alias="totalAmount"), pydantic.Field(alias="totalAmount")
    ]
    net_amount: typing_extensions.Annotated[float, FieldMetadata(alias="netAmount"), pydantic.Field(alias="netAmount")]
    fee_amount: typing_extensions.Annotated[
        FeeAmount, FieldMetadata(alias="feeAmount"), pydantic.Field(alias="feeAmount")
    ]
    settlement_status: typing_extensions.Annotated[
        SettlementStatus, FieldMetadata(alias="settlementStatus"), pydantic.Field(alias="settlementStatus")
    ]
    operation: Operation
    response_data: typing_extensions.Annotated[
        TransactionDetailResponseData, FieldMetadata(alias="responseData"), pydantic.Field(alias="responseData")
    ]
    source: Source
    schedule_reference: typing_extensions.Annotated[
        int, FieldMetadata(alias="scheduleReference"), pydantic.Field(alias="scheduleReference")
    ]
    org_id: typing_extensions.Annotated[Orgid, FieldMetadata(alias="orgId"), pydantic.Field(alias="orgId")]
    refund_id: typing_extensions.Annotated[RefundId, FieldMetadata(alias="refundId"), pydantic.Field(alias="refundId")]
    returned_id: typing_extensions.Annotated[
        ReturnedId, FieldMetadata(alias="returnedId"), pydantic.Field(alias="returnedId")
    ]
    chargeback_id: typing_extensions.Annotated[
        ChargebackId, FieldMetadata(alias="chargebackId"), pydantic.Field(alias="chargebackId")
    ]
    retrieval_id: typing_extensions.Annotated[
        RetrievalId, FieldMetadata(alias="retrievalId"), pydantic.Field(alias="retrievalId")
    ]
    trans_additional_data: typing_extensions.Annotated[
        typing.Optional[typing.Any],
        FieldMetadata(alias="transAdditionalData"),
        pydantic.Field(alias="transAdditionalData"),
    ] = None
    invoice_data: typing_extensions.Annotated[
        TransactionDetailInvoiceData, FieldMetadata(alias="invoiceData"), pydantic.Field(alias="invoiceData")
    ]
    entrypage_id: typing_extensions.Annotated[
        EntrypageId, FieldMetadata(alias="entrypageId"), pydantic.Field(alias="entrypageId")
    ]
    external_paypoint_id: typing_extensions.Annotated[
        ExternalPaypointId, FieldMetadata(alias="externalPaypointID"), pydantic.Field(alias="externalPaypointID")
    ]
    is_validated_ach: typing_extensions.Annotated[
        bool, FieldMetadata(alias="isValidatedACH"), pydantic.Field(alias="isValidatedACH")
    ]
    transaction_time: typing_extensions.Annotated[
        str, FieldMetadata(alias="transactionTime"), pydantic.Field(alias="transactionTime")
    ]
    customer: TransactionDetailCustomer
    split_funding_instructions: typing_extensions.Annotated[
        typing.Optional[SplitFunding],
        FieldMetadata(alias="splitFundingInstructions"),
        pydantic.Field(alias="splitFundingInstructions"),
    ] = None
    cfee_transactions: typing_extensions.Annotated[
        typing.List[QueryCFeeTransaction],
        FieldMetadata(alias="cfeeTransactions"),
        pydantic.Field(alias="cfeeTransactions"),
    ]
    transaction_events: typing_extensions.Annotated[
        typing.List[TransactionDetailEvent],
        FieldMetadata(alias="transactionEvents"),
        pydantic.Field(alias="transactionEvents"),
    ]
    pending_fee_amount: typing_extensions.Annotated[
        PendingFeeAmount, FieldMetadata(alias="pendingFeeAmount"), pydantic.Field(alias="pendingFeeAmount")
    ]
    risk_flagged: typing_extensions.Annotated[
        RiskFlagged, FieldMetadata(alias="riskFlagged"), pydantic.Field(alias="riskFlagged")
    ]
    risk_flagged_on: typing_extensions.Annotated[
        RiskFlaggedOn, FieldMetadata(alias="riskFlaggedOn"), pydantic.Field(alias="riskFlaggedOn")
    ]
    risk_status: typing_extensions.Annotated[
        RiskStatus, FieldMetadata(alias="riskStatus"), pydantic.Field(alias="riskStatus")
    ]
    risk_reason: typing_extensions.Annotated[
        RiskReason, FieldMetadata(alias="riskReason"), pydantic.Field(alias="riskReason")
    ]
    risk_action: typing_extensions.Annotated[
        RiskAction, FieldMetadata(alias="riskAction"), pydantic.Field(alias="riskAction")
    ]
    risk_action_code: typing_extensions.Annotated[
        RiskActionCode, FieldMetadata(alias="riskActionCode"), pydantic.Field(alias="riskActionCode")
    ]
    device_id: typing_extensions.Annotated[Device, FieldMetadata(alias="deviceId"), pydantic.Field(alias="deviceId")]
    ach_sec_code: typing_extensions.Annotated[
        AchSecCode, FieldMetadata(alias="achSecCode"), pydantic.Field(alias="achSecCode")
    ]
    ach_holder_type: typing_extensions.Annotated[
        AchHolderType, FieldMetadata(alias="achHolderType"), pydantic.Field(alias="achHolderType")
    ]
    ip_address: typing_extensions.Annotated[
        IpAddress, FieldMetadata(alias="ipAddress"), pydantic.Field(alias="ipAddress")
    ]
    is_same_day_ach: typing_extensions.Annotated[
        bool, FieldMetadata(alias="isSameDayACH"), pydantic.Field(alias="isSameDayACH")
    ]
    wallet_type: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="walletType"), pydantic.Field(alias="walletType")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class CapturePaymentDetails(UniversalBaseModel):
    total_amount: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="totalAmount"),
        pydantic.Field(
            alias="totalAmount",
            description="Total amount to be captured, including the `serviceFee` amount. The amount can't be greater the original\ntotal amount of the transaction, and can't be more than 15% lower than the original amount.",
        ),
    ]
    service_fee: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="serviceFee"),
        pydantic.Field(alias="serviceFee", description="Service fee to capture for the transaction."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.billing_fee_detail import BillingFeeDetail
from .transfer_out_detail_bill import TransferOutDetailBill
from .transfer_out_detail_check_data import TransferOutDetailCheckData
from .transfer_out_detail_event import TransferOutDetailEvent
from .transfer_out_detail_payment_data import TransferOutDetailPaymentData
from .transfer_out_detail_vendor import TransferOutDetailVendor


class TransferOutDetailRecord(UniversalBaseModel):
    """
    A record representing an outbound transfer detail.
    """

    transfer_detail_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="transferDetailId"),
        pydantic.Field(alias="transferDetailId", description="Unique identifier for the transfer detail."),
    ] = None
    transfer_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="transferId"),
        pydantic.Field(alias="transferId", description="The ID of the transfer this detail belongs to."),
    ] = None
    transaction_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="transactionId"),
        pydantic.Field(alias="transactionId", description="The transaction ID in Payabli's system."),
    ] = None
    id_out: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="IdOut"),
        pydantic.Field(alias="IdOut", description="The outbound transaction ID."),
    ] = None
    method: typing.Optional[str] = pydantic.Field(default=None)
    """
    Payment method used.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transaction type (credit or debit).
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    Category of the transaction detail.
    """

    gross_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="grossAmount"),
        pydantic.Field(alias="grossAmount", description="The gross amount of the transaction."),
    ] = None
    returned_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="returnedAmount"),
        pydantic.Field(alias="returnedAmount", description="Amount returned."),
    ] = None
    refund_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="refundAmount"),
        pydantic.Field(alias="refundAmount", description="Amount refunded."),
    ] = None
    hold_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="holdAmount"),
        pydantic.Field(alias="holdAmount", description="Amount being held."),
    ] = None
    released_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="releasedAmount"),
        pydantic.Field(alias="releasedAmount", description="Amount released."),
    ] = None
    billing_fees_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="billingFeesAmount"),
        pydantic.Field(alias="billingFeesAmount", description="Billing fees amount."),
    ] = None
    adjustments_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="adjustmentsAmount"),
        pydantic.Field(alias="adjustmentsAmount", description="Adjustments amount."),
    ] = None
    net_transfer_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="netTransferAmount"),
        pydantic.Field(alias="netTransferAmount", description="Net transfer amount after deductions."),
    ] = None
    billing_fees_details: typing_extensions.Annotated[
        typing.Optional[typing.List[BillingFeeDetail]],
        FieldMetadata(alias="billingFeesDetails"),
        pydantic.Field(alias="billingFeesDetails", description="Detailed breakdown of billing fees."),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="CreatedAt"),
        pydantic.Field(alias="CreatedAt", description="Date and time the record was created."),
    ] = None
    comments: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Comments"),
        pydantic.Field(alias="Comments", description="Comments on the transfer detail."),
    ] = None
    vendor: typing_extensions.Annotated[
        typing.Optional[TransferOutDetailVendor],
        FieldMetadata(alias="Vendor"),
        pydantic.Field(alias="Vendor", description="Vendor information."),
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaypointDbaname"),
        pydantic.Field(alias="PaypointDbaname", description="DBA name of the paypoint."),
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaypointLegalname"),
        pydantic.Field(alias="PaypointLegalname", description="Legal name of the paypoint."),
    ] = None
    paypoint_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="PaypointId"),
        pydantic.Field(alias="PaypointId", description="ID of the paypoint."),
    ] = None
    status: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="Status"),
        pydantic.Field(alias="Status", description="Status of the transfer detail."),
    ] = None
    payment_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentId"),
        pydantic.Field(alias="PaymentId", description="Payment ID."),
    ] = None
    trans_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="TransId"),
        pydantic.Field(alias="TransId", description="Transaction ID."),
    ] = None
    trans_status: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="TransStatus"),
        pydantic.Field(alias="TransStatus", description="Transaction status."),
    ] = None
    trans_status_detail: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="TransStatusDetail"),
        pydantic.Field(alias="TransStatusDetail", description="Detailed transaction status."),
    ] = None
    trans_status_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="TransStatusName"),
        pydantic.Field(alias="TransStatusName", description="Name of the transaction status."),
    ] = None
    trans_status_category: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="TransStatusCategory"),
        pydantic.Field(alias="TransStatusCategory", description="Category of the transaction status."),
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="LastUpdated"),
        pydantic.Field(alias="LastUpdated", description="Date and time the record was last updated."),
    ] = None
    total_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="TotalAmount"),
        pydantic.Field(alias="TotalAmount", description="Total amount of the transaction."),
    ] = None
    net_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="NetAmount"),
        pydantic.Field(alias="NetAmount", description="Net amount of the transaction."),
    ] = None
    fee_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="FeeAmount"),
        pydantic.Field(alias="FeeAmount", description="Fee amount for the transaction."),
    ] = None
    source: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Source"),
        pydantic.Field(alias="Source", description="Source of the transaction."),
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="ParentOrgName"),
        pydantic.Field(alias="ParentOrgName", description="Name of the parent organization."),
    ] = None
    parent_org_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ParentOrgId"),
        pydantic.Field(alias="ParentOrgId", description="ID of the parent organization."),
    ] = None
    batch_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BatchNumber"),
        pydantic.Field(alias="BatchNumber", description="Batch number for the transfer."),
    ] = None
    payment_status: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentStatus"),
        pydantic.Field(alias="PaymentStatus", description="Status of the payment."),
    ] = None
    payment_method: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentMethod"),
        pydantic.Field(alias="PaymentMethod", description="Payment method used."),
    ] = None
    card_token: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="CardToken"),
        pydantic.Field(alias="CardToken", description="Token for the card used."),
    ] = None
    check_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="CheckNumber"),
        pydantic.Field(alias="CheckNumber", description="Check number if paid by check."),
    ] = None
    check_data: typing_extensions.Annotated[
        typing.Optional[TransferOutDetailCheckData],
        FieldMetadata(alias="CheckData"),
        pydantic.Field(alias="CheckData", description="Check data if paid by check."),
    ] = None
    payment_data: typing_extensions.Annotated[
        typing.Optional[TransferOutDetailPaymentData],
        FieldMetadata(alias="PaymentData"),
        pydantic.Field(alias="PaymentData", description="Payment data for the transaction."),
    ] = None
    bills: typing_extensions.Annotated[
        typing.Optional[typing.List[TransferOutDetailBill]],
        FieldMetadata(alias="Bills"),
        pydantic.Field(alias="Bills", description="Bills associated with the transfer."),
    ] = None
    events: typing_extensions.Annotated[
        typing.Optional[typing.List[TransferOutDetailEvent]],
        FieldMetadata(alias="Events"),
        pydantic.Field(alias="Events", description="Events associated with the transfer."),
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID", description="External paypoint ID."),
    ] = None
    entry_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="EntryName"),
        pydantic.Field(alias="EntryName", description="Entry name for the paypoint."),
    ] = None
    gateway: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Gateway"),
        pydantic.Field(alias="Gateway", description="Gateway used for the transaction."),
    ] = None
    batch_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="BatchId"),
        pydantic.Field(alias="BatchId", description="ID of the batch."),
    ] = None
    has_vcard_transactions: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="HasVcardTransactions"),
        pydantic.Field(alias="HasVcardTransactions", description="Whether the transfer has virtual card transactions."),
    ] = None
    is_same_day_ach: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="IsSameDayACH"),
        pydantic.Field(alias="IsSameDayACH", description="Whether this is a same-day ACH transaction."),
    ] = None
    schedule_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ScheduleId"),
        pydantic.Field(alias="ScheduleId", description="ID of the schedule if applicable."),
    ] = None
    settlement_status: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="SettlementStatus"),
        pydantic.Field(alias="SettlementStatus", description="Settlement status."),
    ] = None
    settlement_status_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="SettlementStatusName"),
        pydantic.Field(alias="SettlementStatusName", description="Name of the settlement status."),
    ] = None
    settlement_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="SettlementDate"),
        pydantic.Field(alias="SettlementDate", description="Date of settlement."),
    ] = None
    risk_flagged: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="RiskFlagged"),
        pydantic.Field(alias="RiskFlagged", description="Whether the transaction was flagged for risk."),
    ] = None
    risk_flagged_on: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="RiskFlaggedOn"),
        pydantic.Field(alias="RiskFlaggedOn", description="Date and time the transaction was flagged."),
    ] = None
    risk_status: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="RiskStatus"),
        pydantic.Field(alias="RiskStatus", description="Risk status of the transaction."),
    ] = None
    risk_reason: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="RiskReason"),
        pydantic.Field(alias="RiskReason", description="Reason for the risk flag."),
    ] = None
    risk_action: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="RiskAction"),
        pydantic.Field(alias="RiskAction", description="Action taken for risk."),
    ] = None
    risk_action_code: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="RiskActionCode"),
        pydantic.Field(alias="RiskActionCode", description="Code for the risk action."),
    ] = None
    payout_program: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PayoutProgram"),
        pydantic.Field(alias="PayoutProgram", description="Payout program used."),
    ] = None
    ach_trace_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="AchTraceNumber"),
        pydantic.Field(alias="AchTraceNumber", description="ACH trace number."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.order_id import OrderId
from ...types.orderdescription import Orderdescription
from ...types.source import Source


class TransactionDetailPaymentDetails(UniversalBaseModel):
    """
    Detailed breakdown of payment amounts and identifiers
    """

    total_amount: typing_extensions.Annotated[
        float, FieldMetadata(alias="totalAmount"), pydantic.Field(alias="totalAmount")
    ]
    service_fee: typing_extensions.Annotated[
        float, FieldMetadata(alias="serviceFee"), pydantic.Field(alias="serviceFee")
    ]
    check_number: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="checkNumber"), pydantic.Field(alias="checkNumber")
    ] = None
    check_image: typing_extensions.Annotated[
        typing.Optional[typing.Any], FieldMetadata(alias="checkImage"), pydantic.Field(alias="checkImage")
    ] = None
    check_unique_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="checkUniqueId"), pydantic.Field(alias="checkUniqueId")
    ]
    currency: str
    order_description: typing_extensions.Annotated[
        typing.Optional[Orderdescription],
        FieldMetadata(alias="orderDescription"),
        pydantic.Field(alias="orderDescription"),
    ] = None
    order_id: typing_extensions.Annotated[
        typing.Optional[OrderId], FieldMetadata(alias="orderId"), pydantic.Field(alias="orderId")
    ] = None
    order_id_alternative: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="orderIdAlternative"), pydantic.Field(alias="orderIdAlternative")
    ] = None
    payment_description: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="paymentDescription"), pydantic.Field(alias="paymentDescription")
    ] = None
    group_number: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="groupNumber"), pydantic.Field(alias="groupNumber")
    ] = None
    source: typing.Optional[Source] = None
    payabli_trans_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="payabliTransId"), pydantic.Field(alias="payabliTransId")
    ] = None
    unbundled: typing.Optional[typing.Any] = None
    categories: typing.List[typing.Any]
    split_funding: typing_extensions.Annotated[
        typing.List[typing.Any], FieldMetadata(alias="splitFunding"), pydantic.Field(alias="splitFunding")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TransferOutEventData(UniversalBaseModel):
    """
    Event data associated with an outbound transfer.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the event.
    """

    event_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="eventTime"),
        pydantic.Field(alias="eventTime", description="The time the event occurred."),
    ] = None
    ref_data: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="refData"),
        pydantic.Field(alias="refData", description="Reference data for the event."),
    ] = None
    extra_data: typing_extensions.Annotated[
        typing.Optional[typing.Any],
        FieldMetadata(alias="extraData"),
        pydantic.Field(
            alias="extraData", description="Additional event data, which may contain detailed transaction information."
        ),
    ] = None
    source: typing.Optional[str] = pydantic.Field(default=None)
    """
    The source of the event.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .key_value_duo import KeyValueDuo
from .notification_standard_request_content_event_type import NotificationStandardRequestContentEventType


class NotificationStandardRequestContent(UniversalBaseModel):
    event_type: typing_extensions.Annotated[
        typing.Optional[NotificationStandardRequestContentEventType],
        FieldMetadata(alias="eventType"),
        pydantic.Field(alias="eventType", description="The notification's event name."),
    ] = None
    internal_data: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValueDuo]],
        FieldMetadata(alias="internalData"),
        pydantic.Field(
            alias="internalData",
            description="Array of pairs key:value to insert in request body to target in **method** = *web*.",
        ),
    ] = None
    transaction_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="transactionId"),
        pydantic.Field(
            alias="transactionId", description="Used internally to reference the entity or object generating the event."
        ),
    ] = None
    web_header_parameters: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValueDuo]],
        FieldMetadata(alias="webHeaderParameters"),
        pydantic.Field(
            alias="webHeaderParameters",
            description="Array of pairs key:value to insert in header of request to target in **method** = *web*.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .associated_vendor import AssociatedVendor
from .created_at import CreatedAt
from .customer_data import CustomerData
from .dbaname import Dbaname
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId
from .last_modified import LastModified
from .legalname import Legalname
from .org_parent_name import OrgParentName


class VCardRecord(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import AssociatedVendor, CustomerData, VCardRecord

    VCardRecord(
        vcard_sent=True,
        card_token="vcrd_5Ty8NrBzXjKuqHm9DwElfP",
        card_number="44XX XXXX XXXX 1234",
        cvc="XXX",
        expiration_date="2025-12",
        status="Active",
        amount=500.0,
        current_balance=375.25,
        expense_limit=100.0,
        expense_limit_period="monthly",
        max_number_of_uses=10,
        current_number_of_uses=3,
        exact_amount=False,
        mcc="5812",
        tcc="T01",
        misc_1="Invoice #12345",
        misc_2="Project: Office Supplies",
        date_created=datetime.datetime.fromisoformat(
            "2023-01-15 09:30:00+00:00",
        ),
        date_modified=datetime.datetime.fromisoformat(
            "2023-02-20 14:15:22+00:00",
        ),
        associated_vendor=AssociatedVendor(
            vendor_number="V-12345",
            name_1="Office Supply Co.",
            ein="XXXXX6789",
            email="billing@officesupply.example.com",
            vendor_id=1542,
        ),
        associated_customer=CustomerData(
            firstname="Acme",
            lastname="Corporation",
        ),
        paypoint_dbaname="Global Factory LLC",
        paypoint_legalname="Global Factory LLC",
        paypoint_entryname="4872acb376a",
        external_paypoint_id="pay-10",
        parent_org_name="SupplyPro",
        paypoint_id=236,
    )
    """

    vcard_sent: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="vcardSent"),
        pydantic.Field(alias="vcardSent", description="When `true`, the vCard has been sent."),
    ] = None
    card_token: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="cardToken"), pydantic.Field(alias="cardToken")
    ] = None
    card_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="cardNumber"),
        pydantic.Field(alias="cardNumber", description="The vCard number."),
    ] = None
    cvc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The vCard CVC number.
    """

    expiration_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="expirationDate"),
        pydantic.Field(
            alias="expirationDate",
            description="Expiration date in format YYYY-MM-DD. The minimum time to expire is 3 months, maximum is 3 years. If not provided, the default is 6 months.",
        ),
    ] = None
    status: typing.Optional[str] = None
    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    The vCard amount.
    """

    current_balance: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="currentBalance"),
        pydantic.Field(alias="currentBalance", description="The vCard's current balance."),
    ] = None
    expense_limit: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="expenseLimit"), pydantic.Field(alias="expenseLimit")
    ] = None
    expense_limit_period: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="expenseLimitPeriod"), pydantic.Field(alias="expenseLimitPeriod")
    ] = None
    max_number_of_uses: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="maxNumberOfUses"), pydantic.Field(alias="maxNumberOfUses")
    ] = None
    current_number_of_uses: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="currentNumberOfUses"), pydantic.Field(alias="currentNumberOfUses")
    ] = None
    exact_amount: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="exactAmount"), pydantic.Field(alias="exactAmount")
    ] = None
    mcc: typing.Optional[str] = pydantic.Field(default=None)
    """
    MCC assigned to vCard.
    """

    tcc: typing.Optional[str] = pydantic.Field(default=None)
    """
    TCC assigned to vCard.
    """

    misc_1: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="misc1"), pydantic.Field(alias="misc1", description="Custom field 1.")
    ] = None
    misc_2: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="misc2"), pydantic.Field(alias="misc2", description="Custom field 2.")
    ] = None
    date_created: typing_extensions.Annotated[
        typing.Optional[CreatedAt], FieldMetadata(alias="dateCreated"), pydantic.Field(alias="dateCreated")
    ] = None
    date_modified: typing_extensions.Annotated[
        typing.Optional[LastModified], FieldMetadata(alias="dateModified"), pydantic.Field(alias="dateModified")
    ] = None
    associated_vendor: typing_extensions.Annotated[
        typing.Optional[AssociatedVendor],
        FieldMetadata(alias="associatedVendor"),
        pydantic.Field(alias="associatedVendor"),
    ] = None
    associated_customer: typing_extensions.Annotated[
        typing.Optional[CustomerData],
        FieldMetadata(alias="associatedCustomer"),
        pydantic.Field(alias="associatedCustomer"),
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName"), pydantic.Field(alias="ParentOrgName")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[
        typing.Optional[Dbaname],
        FieldMetadata(alias="PaypointDbaname"),
        pydantic.Field(alias="PaypointDbaname", description="The paypoint's DBA name."),
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname],
        FieldMetadata(alias="PaypointLegalname"),
        pydantic.Field(alias="PaypointLegalname", description="The paypoint's legal name."),
    ] = None
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield],
        FieldMetadata(alias="PaypointEntryname"),
        pydantic.Field(alias="PaypointEntryname", description="The paypoint's entry name (entrypoint)."),
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None
    paypoint_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="paypointId"),
        pydantic.Field(alias="paypointId", description="The paypoint's unique identifier."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .customer_id import CustomerId
from .customer_number_nullable import CustomerNumberNullable
from .customer_query_records_customer_consent import CustomerQueryRecordsCustomerConsent
from .customer_status import CustomerStatus
from .customer_summary_record import CustomerSummaryRecord
from .dbaname import Dbaname
from .email import Email
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId
from .identifierfields import Identifierfields
from .legalname import Legalname
from .method_query_records import MethodQueryRecords
from .mfa import Mfa
from .mfa_mode import MfaMode
from .org_parent_id import OrgParentId
from .org_parent_name import OrgParentName
from .page_identifier import PageIdentifier
from .shippingaddress import Shippingaddress
from .shippingaddressadditional import Shippingaddressadditional
from .shippingcity import Shippingcity
from .shippingcountry import Shippingcountry
from .shippingstate import Shippingstate
from .shippingzip import Shippingzip
from .subscription_query_records import SubscriptionQueryRecords
from .timezone import Timezone


class CustomerQueryRecords(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import (
        BinData,
        CustomerQueryRecords,
        CustomerQueryRecordsCustomerConsent,
        CustomerQueryRecordsCustomerConsentECommunication,
        CustomerQueryRecordsCustomerConsentSms,
        CustomerSummaryRecord,
        GeneralEvents,
        MethodQueryRecords,
        PaymentDetail,
        QueryPaymentData,
        SubscriptionQueryRecords,
        TransactionQueryRecords,
    )

    CustomerQueryRecords(
        customer_id=4440,
        customer_number="3456-7645A",
        customer_username="myusername",
        customer_status=1,
        company="AA LLC",
        firstname="John",
        lastname="Smith",
        phone="1234567890",
        email="example@email.com",
        address="3245 Main St",
        address_1="STE 900",
        city="Miami",
        state="FL",
        zip="77777",
        country="US",
        shipping_address="123 Walnut St",
        shipping_address_1="STE 900",
        shipping_city="Johnson City",
        shipping_state="TN",
        shipping_zip="37619",
        shipping_country="US",
        balance=1.1,
        time_zone=-5,
        mfa=False,
        mfa_mode=0,
        sn_provider="facebook",
        sn_identifier="6677fgttyudd999",
        sn_data="",
        last_updated=datetime.datetime.fromisoformat(
            "2021-06-16 05:00:00+00:00",
        ),
        created=datetime.datetime.fromisoformat(
            "2021-06-10 05:00:00+00:00",
        ),
        additional_fields={"property1": "string", "property2": "string"},
        identifier_fields=["email"],
        subscriptions=[
            SubscriptionQueryRecords(
                created_at=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
                end_date=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                entrypage_id=0,
                external_paypoint_id="Paypoint-100",
                fee_amount=3.0,
                frequency="monthly",
                id_sub=396,
                last_run=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                last_updated=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
                left_cycles=15,
                method="card",
                net_amount=3762.87,
                next_date=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                parent_org_name="PropertyManager Pro",
                payment_data=QueryPaymentData(
                    payment_details=PaymentDetail(
                        total_amount=100.0,
                    ),
                ),
                paypoint_dbaname="Sunshine Gutters",
                paypoint_entryname="d193cf9a46",
                paypoint_id=255,
                paypoint_legalname="Sunshine Services, LLC",
                plan_id=0,
                source="api",
                start_date=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                sub_events=[
                    GeneralEvents(
                        description="TransferCreated",
                        event_time=datetime.datetime.fromisoformat(
                            "2023-07-05 22:31:06+00:00",
                        ),
                    )
                ],
                sub_status=1,
                total_amount=103.0,
                total_cycles=24,
                until_cancelled=True,
            )
        ],
        stored_methods=[
            MethodQueryRecords(
                bin="411111",
                bin_data=BinData(
                    bin_matched_length="6",
                    bin_card_brand="Visa",
                    bin_card_type="Credit",
                    bin_card_category="PLATINUM",
                    bin_card_issuer="Bank of Example",
                    bin_card_issuer_country="United States",
                    bin_card_issuer_country_code_a_2="US",
                    bin_card_issuer_country_number="840",
                    bin_card_is_regulated="false",
                    bin_card_use_category="Consumer",
                    bin_card_issuer_country_code_a_3="USA",
                ),
                descriptor="visa",
                exp_date="1227",
                holder_name="Chad Mercia",
                id_pmethod="6edcbb56-9c0e-4003-b3d1-99abf149ba0e",
                last_updated=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
                masked_account="4XXXXXXXX1111",
                method="card",
            )
        ],
        customer_summary=CustomerSummaryRecord(
            numberof_transactions=30,
            recent_transactions=[
                TransactionQueryRecords(
                    entrypage_id=0,
                    fee_amount=1.0,
                    payor_id=1551,
                    paypoint_id=226,
                    settlement_status=2,
                    total_amount=30.22,
                    trans_status=1,
                )
            ],
            total_amount_transactions=1500.0,
            total_net_amount_transactions=1500.0,
        ),
        paypoint_legalname="Sunshine Services, LLC",
        paypoint_dbaname="Sunshine Gutters",
        parent_org_name="PropertyManager Pro",
        parent_org_id=123,
        paypoint_entryname="d193cf9a46",
        pageidentifier="null",
        external_paypoint_id="Paypoint-100",
        customer_consent=CustomerQueryRecordsCustomerConsent(
            e_communication=CustomerQueryRecordsCustomerConsentECommunication(
                status=1,
                updated_at=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
            ),
            sms=CustomerQueryRecordsCustomerConsentSms(
                status=1,
                updated_at=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
            ),
        ),
    )
    """

    customer_id: typing_extensions.Annotated[
        typing.Optional[CustomerId], FieldMetadata(alias="customerId"), pydantic.Field(alias="customerId")
    ] = None
    customer_number: typing_extensions.Annotated[
        typing.Optional[CustomerNumberNullable],
        FieldMetadata(alias="customerNumber"),
        pydantic.Field(alias="customerNumber"),
    ] = None
    customer_username: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="customerUsername"),
        pydantic.Field(alias="customerUsername", description="Username for customer."),
    ] = None
    customer_status: typing_extensions.Annotated[
        typing.Optional[CustomerStatus], FieldMetadata(alias="customerStatus"), pydantic.Field(alias="customerStatus")
    ] = None
    company: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Company"),
        pydantic.Field(alias="Company", description="Company name."),
    ] = None
    firstname: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Firstname"),
        pydantic.Field(alias="Firstname", description="Customer first name."),
    ] = None
    lastname: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Lastname"),
        pydantic.Field(alias="Lastname", description="Customer last name."),
    ] = None
    phone: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Phone"),
        pydantic.Field(alias="Phone", description="Customer phone number."),
    ] = None
    email: typing_extensions.Annotated[
        typing.Optional[Email],
        FieldMetadata(alias="Email"),
        pydantic.Field(alias="Email", description="Customer email address."),
    ] = None
    address: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Address"),
        pydantic.Field(alias="Address", description="Customer address."),
    ] = None
    address_1: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Address1"),
        pydantic.Field(alias="Address1", description="Additional line for customer address."),
    ] = None
    city: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="City"), pydantic.Field(alias="City", description="Customer city.")
    ] = None
    state: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="State"), pydantic.Field(alias="State", description="Customer state.")
    ] = None
    zip: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Zip"),
        pydantic.Field(alias="Zip", description="Customer postal code."),
    ] = None
    country: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Country"),
        pydantic.Field(alias="Country", description="Customer country."),
    ] = None
    shipping_address: typing_extensions.Annotated[
        typing.Optional[Shippingaddress],
        FieldMetadata(alias="ShippingAddress"),
        pydantic.Field(alias="ShippingAddress"),
    ] = None
    shipping_address_1: typing_extensions.Annotated[
        typing.Optional[Shippingaddressadditional],
        FieldMetadata(alias="ShippingAddress1"),
        pydantic.Field(alias="ShippingAddress1"),
    ] = None
    shipping_city: typing_extensions.Annotated[
        typing.Optional[Shippingcity], FieldMetadata(alias="ShippingCity"), pydantic.Field(alias="ShippingCity")
    ] = None
    shipping_state: typing_extensions.Annotated[
        typing.Optional[Shippingstate], FieldMetadata(alias="ShippingState"), pydantic.Field(alias="ShippingState")
    ] = None
    shipping_zip: typing_extensions.Annotated[
        typing.Optional[Shippingzip], FieldMetadata(alias="ShippingZip"), pydantic.Field(alias="ShippingZip")
    ] = None
    shipping_country: typing_extensions.Annotated[
        typing.Optional[Shippingcountry],
        FieldMetadata(alias="ShippingCountry"),
        pydantic.Field(alias="ShippingCountry"),
    ] = None
    balance: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="Balance"),
        pydantic.Field(alias="Balance", description="Customer balance."),
    ] = None
    time_zone: typing_extensions.Annotated[
        typing.Optional[Timezone], FieldMetadata(alias="TimeZone"), pydantic.Field(alias="TimeZone")
    ] = None
    mfa: typing_extensions.Annotated[typing.Optional[Mfa], FieldMetadata(alias="MFA"), pydantic.Field(alias="MFA")] = (
        None
    )
    mfa_mode: typing_extensions.Annotated[
        typing.Optional[MfaMode], FieldMetadata(alias="MFAMode"), pydantic.Field(alias="MFAMode")
    ] = None
    sn_provider: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snProvider"),
        pydantic.Field(
            alias="snProvider",
            description="Social network linked to customer. Possible values:\n\n- `facebook`\n\n- `google`\n\n- `twitter`\n\n- `microsoft`",
        ),
    ] = None
    sn_identifier: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snIdentifier"),
        pydantic.Field(alias="snIdentifier", description="Identifier or token for customer in linked social network."),
    ] = None
    sn_data: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snData"),
        pydantic.Field(
            alias="snData", description="Additional data provided by the social network related to the customer."
        ),
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="LastUpdated"),
        pydantic.Field(alias="LastUpdated", description="Date and time of last update."),
    ] = None
    created: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="Created"),
        pydantic.Field(alias="Created", description="Date and time created."),
    ] = None
    additional_fields: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[str]]],
        FieldMetadata(alias="AdditionalFields"),
        pydantic.Field(alias="AdditionalFields", description="List of additional custom fields in format key:value."),
    ] = None
    identifier_fields: typing_extensions.Annotated[
        typing.Optional[Identifierfields],
        FieldMetadata(alias="IdentifierFields"),
        pydantic.Field(alias="IdentifierFields"),
    ] = None
    subscriptions: typing_extensions.Annotated[
        typing.Optional[typing.List[SubscriptionQueryRecords]],
        FieldMetadata(alias="Subscriptions"),
        pydantic.Field(alias="Subscriptions", description="List of subscriptions associated to the customer."),
    ] = None
    stored_methods: typing_extensions.Annotated[
        typing.Optional[typing.List[MethodQueryRecords]],
        FieldMetadata(alias="StoredMethods"),
        pydantic.Field(alias="StoredMethods", description="List of payment methods associated to the customer."),
    ] = None
    customer_summary: typing_extensions.Annotated[
        typing.Optional[CustomerSummaryRecord],
        FieldMetadata(alias="customerSummary"),
        pydantic.Field(alias="customerSummary"),
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname],
        FieldMetadata(alias="PaypointLegalname"),
        pydantic.Field(alias="PaypointLegalname", description="Paypoint legal name."),
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[
        typing.Optional[Dbaname],
        FieldMetadata(alias="PaypointDbaname"),
        pydantic.Field(alias="PaypointDbaname", description="Paypoint DBA name."),
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName"), pydantic.Field(alias="ParentOrgName")
    ] = None
    parent_org_id: typing_extensions.Annotated[
        typing.Optional[OrgParentId], FieldMetadata(alias="ParentOrgId"), pydantic.Field(alias="ParentOrgId")
    ] = None
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield],
        FieldMetadata(alias="PaypointEntryname"),
        pydantic.Field(alias="PaypointEntryname"),
    ] = None
    pageidentifier: typing.Optional[PageIdentifier] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None
    customer_consent: typing_extensions.Annotated[
        typing.Optional[CustomerQueryRecordsCustomerConsent],
        FieldMetadata(alias="customerConsent"),
        pydantic.Field(alias="customerConsent"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TransferOutBankAccount(UniversalBaseModel):
    """
    Bank account information for an outbound transfer.
    """

    account_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="accountNumber"),
        pydantic.Field(alias="accountNumber", description="The masked bank account number."),
    ] = None
    routing_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="routingNumber"),
        pydantic.Field(alias="routingNumber", description="The bank routing number."),
    ] = None
    bank_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="bankName"),
        pydantic.Field(alias="bankName", description="The bank name."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.additional_data_map import AdditionalDataMap
from ...types.bill_events import BillEvents
from ...types.bill_item import BillItem
from ...types.bill_options import BillOptions
from ...types.created_at import CreatedAt
from ...types.customer_id import CustomerId
from ...types.datenullable import Datenullable
from ...types.datetime_nullable import DatetimeNullable
from ...types.dbaname import Dbaname
from ...types.discount import Discount
from ...types.documents_ref import DocumentsRef
from ...types.duty_amount import DutyAmount
from ...types.email import Email
from ...types.entrypointfield import Entrypointfield
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.freight_amount import FreightAmount
from ...types.frequency import Frequency
from ...types.invoice_amount import InvoiceAmount
from ...types.invoice_number import InvoiceNumber
from ...types.invoice_type import InvoiceType
from ...types.invoicestatus import Invoicestatus
from ...types.org_parent_name import OrgParentName
from ...types.paylink_id import PaylinkId
from ...types.payor_data_response import PayorDataResponse
from ...types.paypoint_id import PaypointId
from ...types.purchase_order import PurchaseOrder
from ...types.shipping_from_zip import ShippingFromZip
from ...types.shippingaddress import Shippingaddress
from ...types.shippingaddressadditional import Shippingaddressadditional
from ...types.shippingcity import Shippingcity
from ...types.shippingcountry import Shippingcountry
from ...types.shippingstate import Shippingstate
from ...types.shippingzip import Shippingzip
from ...types.summary_commodity_code import SummaryCommodityCode
from ...types.tax import Tax
from ...types.terms import Terms
from ...types.terms_conditions import TermsConditions
from .invoice_id import InvoiceId
from .invoice_paid_amount import InvoicePaidAmount


class GetInvoiceRecord(UniversalBaseModel):
    invoice_id: typing_extensions.Annotated[
        InvoiceId, FieldMetadata(alias="invoiceId"), pydantic.Field(alias="invoiceId")
    ]
    customer_id: typing_extensions.Annotated[
        CustomerId, FieldMetadata(alias="customerId"), pydantic.Field(alias="customerId")
    ]
    paypoint_id: typing_extensions.Annotated[
        PaypointId, FieldMetadata(alias="paypointId"), pydantic.Field(alias="paypointId")
    ]
    invoice_number: typing_extensions.Annotated[
        InvoiceNumber, FieldMetadata(alias="invoiceNumber"), pydantic.Field(alias="invoiceNumber")
    ]
    invoice_date: typing_extensions.Annotated[
        Datenullable, FieldMetadata(alias="invoiceDate"), pydantic.Field(alias="invoiceDate")
    ]
    invoice_due_date: typing_extensions.Annotated[
        Datenullable, FieldMetadata(alias="invoiceDueDate"), pydantic.Field(alias="invoiceDueDate")
    ]
    invoice_sent_date: typing_extensions.Annotated[
        DatetimeNullable, FieldMetadata(alias="invoiceSentDate"), pydantic.Field(alias="invoiceSentDate")
    ]
    invoice_end_date: typing_extensions.Annotated[
        Datenullable, FieldMetadata(alias="invoiceEndDate"), pydantic.Field(alias="invoiceEndDate")
    ]
    last_payment_date: typing_extensions.Annotated[
        DatetimeNullable, FieldMetadata(alias="lastPaymentDate"), pydantic.Field(alias="lastPaymentDate")
    ]
    created_at: typing_extensions.Annotated[
        CreatedAt, FieldMetadata(alias="createdAt"), pydantic.Field(alias="createdAt")
    ]
    invoice_status: typing_extensions.Annotated[
        Invoicestatus, FieldMetadata(alias="invoiceStatus"), pydantic.Field(alias="invoiceStatus")
    ]
    invoice_type: typing_extensions.Annotated[
        InvoiceType, FieldMetadata(alias="invoiceType"), pydantic.Field(alias="invoiceType")
    ]
    frequency: Frequency
    payment_terms: typing_extensions.Annotated[
        Terms, FieldMetadata(alias="paymentTerms"), pydantic.Field(alias="paymentTerms")
    ]
    terms_conditions: typing_extensions.Annotated[
        typing.Optional[TermsConditions],
        FieldMetadata(alias="termsConditions"),
        pydantic.Field(alias="termsConditions"),
    ] = None
    notes: typing.Optional[str] = None
    tax: Tax
    discount: Discount
    invoice_amount: typing_extensions.Annotated[
        InvoiceAmount, FieldMetadata(alias="invoiceAmount"), pydantic.Field(alias="invoiceAmount")
    ]
    invoice_paid_amount: typing_extensions.Annotated[
        InvoicePaidAmount, FieldMetadata(alias="invoicePaidAmount"), pydantic.Field(alias="invoicePaidAmount")
    ]
    freight_amount: typing_extensions.Annotated[
        FreightAmount, FieldMetadata(alias="freightAmount"), pydantic.Field(alias="freightAmount")
    ]
    duty_amount: typing_extensions.Annotated[
        DutyAmount, FieldMetadata(alias="dutyAmount"), pydantic.Field(alias="dutyAmount")
    ]
    purchase_order: typing_extensions.Annotated[
        PurchaseOrder, FieldMetadata(alias="purchaseOrder"), pydantic.Field(alias="purchaseOrder")
    ]
    first_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="firstName"),
        pydantic.Field(alias="firstName", description="First name of the recipient of the invoice."),
    ] = None
    last_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="lastName"),
        pydantic.Field(alias="lastName", description="Last name of the recipient of the invoice."),
    ] = None
    company: typing.Optional[str] = pydantic.Field(default=None)
    """
    Company name of the recipient of the invoice.
    """

    shipping_address_1: typing_extensions.Annotated[
        Shippingaddress, FieldMetadata(alias="shippingAddress1"), pydantic.Field(alias="shippingAddress1")
    ]
    shipping_address_2: typing_extensions.Annotated[
        Shippingaddressadditional, FieldMetadata(alias="shippingAddress2"), pydantic.Field(alias="shippingAddress2")
    ]
    shipping_city: typing_extensions.Annotated[
        Shippingcity, FieldMetadata(alias="shippingCity"), pydantic.Field(alias="shippingCity")
    ]
    shipping_state: typing_extensions.Annotated[
        Shippingstate, FieldMetadata(alias="shippingState"), pydantic.Field(alias="shippingState")
    ]
    shipping_zip: typing_extensions.Annotated[
        Shippingzip, FieldMetadata(alias="shippingZip"), pydantic.Field(alias="shippingZip")
    ]
    shipping_from_zip: typing_extensions.Annotated[
        ShippingFromZip, FieldMetadata(alias="shippingFromZip"), pydantic.Field(alias="shippingFromZip")
    ]
    shipping_country: typing_extensions.Annotated[
        Shippingcountry, FieldMetadata(alias="shippingCountry"), pydantic.Field(alias="shippingCountry")
    ]
    shipping_email: typing_extensions.Annotated[
        Email, FieldMetadata(alias="shippingEmail"), pydantic.Field(alias="shippingEmail")
    ]
    shipping_phone: typing_extensions.Annotated[
        str, FieldMetadata(alias="shippingPhone"), pydantic.Field(alias="shippingPhone")
    ]
    summary_commodity_code: typing_extensions.Annotated[
        SummaryCommodityCode, FieldMetadata(alias="summaryCommodityCode"), pydantic.Field(alias="summaryCommodityCode")
    ]
    items: typing.List[BillItem]
    customer: typing_extensions.Annotated[
        PayorDataResponse, FieldMetadata(alias="Customer"), pydantic.Field(alias="Customer")
    ]
    paylink_id: typing_extensions.Annotated[
        PaylinkId, FieldMetadata(alias="paylinkId"), pydantic.Field(alias="paylinkId")
    ]
    bill_events: typing_extensions.Annotated[
        BillEvents, FieldMetadata(alias="billEvents"), pydantic.Field(alias="billEvents")
    ]
    scheduled_options: typing_extensions.Annotated[
        BillOptions, FieldMetadata(alias="scheduledOptions"), pydantic.Field(alias="scheduledOptions")
    ]
    paypoint_legalname: typing_extensions.Annotated[
        str, FieldMetadata(alias="PaypointLegalname"), pydantic.Field(alias="PaypointLegalname")
    ]
    paypoint_dbaname: typing_extensions.Annotated[
        Dbaname, FieldMetadata(alias="PaypointDbaname"), pydantic.Field(alias="PaypointDbaname")
    ]
    paypoint_entryname: typing_extensions.Annotated[
        Entrypointfield, FieldMetadata(alias="PaypointEntryname"), pydantic.Field(alias="PaypointEntryname")
    ]
    parent_org_name: typing_extensions.Annotated[
        OrgParentName, FieldMetadata(alias="ParentOrgName"), pydantic.Field(alias="ParentOrgName")
    ]
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataMap],
        FieldMetadata(alias="AdditionalData"),
        pydantic.Field(alias="AdditionalData"),
    ] = None
    documents_ref: typing_extensions.Annotated[
        DocumentsRef, FieldMetadata(alias="DocumentsRef"), pydantic.Field(alias="DocumentsRef")
    ]
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.page_identifier import PageIdentifier
from ...types.pagesize import Pagesize
from ...types.totalpages import Totalpages
from ...types.totalrecords import Totalrecords


class BatchDetailResponseSummary(UniversalBaseModel):
    service_fees: typing_extensions.Annotated[
        float, FieldMetadata(alias="serviceFees"), pydantic.Field(alias="serviceFees")
    ]
    transfer_amount: typing_extensions.Annotated[
        float, FieldMetadata(alias="transferAmount"), pydantic.Field(alias="transferAmount")
    ]
    refunds: float
    held_amount: typing_extensions.Annotated[
        float, FieldMetadata(alias="heldAmount"), pydantic.Field(alias="heldAmount")
    ]
    total_records: typing_extensions.Annotated[
        Totalrecords, FieldMetadata(alias="totalRecords"), pydantic.Field(alias="totalRecords")
    ]
    total_amount: typing_extensions.Annotated[
        float, FieldMetadata(alias="totalAmount"), pydantic.Field(alias="totalAmount")
    ]
    total_net_amount: typing_extensions.Annotated[
        float, FieldMetadata(alias="totalNetAmount"), pydantic.Field(alias="totalNetAmount")
    ]
    total_pages: typing_extensions.Annotated[
        Totalpages, FieldMetadata(alias="totalPages"), pydantic.Field(alias="totalPages")
    ]
    page_size: typing_extensions.Annotated[Pagesize, FieldMetadata(alias="pageSize"), pydantic.Field(alias="pageSize")]
    pageidentifier: typing.Optional[PageIdentifier] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.accounting_field import AccountingField
from ...types.additional_data_string import AdditionalDataString
from ...types.bill_approvals import BillApprovals
from ...types.bill_events import BillEvents
from ...types.bill_id import BillId
from ...types.billitems import Billitems
from ...types.billstatus import Billstatus
from ...types.created_at import CreatedAt
from ...types.datenullable import Datenullable
from ...types.dbaname import Dbaname
from ...types.documents_ref import DocumentsRef
from ...types.entity_id import EntityId
from ...types.entrypointfield import Entrypointfield
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.frequency import Frequency
from ...types.last_modified import LastModified
from ...types.legalname import Legalname
from ...types.org_parent_id import OrgParentId
from ...types.org_parent_name import OrgParentName
from ...types.paylink_id import PaylinkId
from ...types.terms import Terms
from ...types.transaction_out_query_record import TransactionOutQueryRecord
from ...types.vendor_data_response import VendorDataResponse


class BillResponseData(UniversalBaseModel):
    id_bill: typing_extensions.Annotated[
        typing.Optional[BillId], FieldMetadata(alias="IdBill"), pydantic.Field(alias="IdBill")
    ] = None
    bill_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillNumber"),
        pydantic.Field(alias="BillNumber", description="Unique identifier for the bill."),
    ] = None
    net_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="NetAmount"),
        pydantic.Field(alias="NetAmount", description="Net amount owed in bill."),
    ] = None
    discount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="Discount"),
        pydantic.Field(alias="Discount", description="Bill discount amount."),
    ] = None
    total_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="TotalAmount"),
        pydantic.Field(alias="TotalAmount", description="Total amount for the bill."),
    ] = None
    bill_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="BillDate"),
        pydantic.Field(alias="BillDate", description="Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY"),
    ] = None
    due_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="DueDate"),
        pydantic.Field(alias="DueDate", description="Due Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY"),
    ] = None
    comments: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Comments"),
        pydantic.Field(
            alias="Comments",
            description="Comments associated with the bill. For managed payables, the character limit is 200. For on demand payouts, the characters limit is 250.",
        ),
    ] = None
    batch_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BatchNumber"),
        pydantic.Field(alias="BatchNumber", description="The batch number that the bill belongs to."),
    ] = None
    bill_items: typing_extensions.Annotated[
        typing.Optional[Billitems],
        FieldMetadata(alias="BillItems"),
        pydantic.Field(alias="BillItems", description="Array of `LineItems` contained in bill."),
    ] = None
    mode: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="Mode"),
        pydantic.Field(
            alias="Mode", description="Bill mode: value `0` for single/one-time bills, `1` for scheduled bills."
        ),
    ] = None
    payment_method: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentMethod"),
        pydantic.Field(alias="PaymentMethod", description="Payment method used for the bill."),
    ] = None
    payment_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentId"),
        pydantic.Field(alias="PaymentId", description="Payment ID associated with the bill."),
    ] = None
    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField],
        FieldMetadata(alias="AccountingField1"),
        pydantic.Field(alias="AccountingField1"),
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField],
        FieldMetadata(alias="AccountingField2"),
        pydantic.Field(alias="AccountingField2"),
    ] = None
    terms: typing_extensions.Annotated[
        typing.Optional[Terms], FieldMetadata(alias="Terms"), pydantic.Field(alias="Terms")
    ] = None
    source: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Source"),
        pydantic.Field(alias="Source", description='The source of the bill, such as "API" or "UI".'),
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString],
        FieldMetadata(alias="AdditionalData"),
        pydantic.Field(alias="AdditionalData"),
    ] = None
    vendor: typing_extensions.Annotated[
        typing.Optional[VendorDataResponse], FieldMetadata(alias="Vendor"), pydantic.Field(alias="Vendor")
    ] = None
    status: typing_extensions.Annotated[
        typing.Optional[Billstatus], FieldMetadata(alias="Status"), pydantic.Field(alias="Status")
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt"), pydantic.Field(alias="CreatedAt")
    ] = None
    end_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="EndDate"),
        pydantic.Field(alias="EndDate", description="End date for scheduled bills. Applied only in `Mode` = 1."),
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[LastModified], FieldMetadata(alias="LastUpdated"), pydantic.Field(alias="LastUpdated")
    ] = None
    frequency: typing_extensions.Annotated[
        typing.Optional[Frequency],
        FieldMetadata(alias="Frequency"),
        pydantic.Field(alias="Frequency", description="Frequency for scheduled bills. Applied only in `Mode` = 1."),
    ] = None
    transaction: typing_extensions.Annotated[
        typing.Optional[TransactionOutQueryRecord],
        FieldMetadata(alias="Transaction"),
        pydantic.Field(alias="Transaction", description="MoneyOut transaction associated to the bill"),
    ] = None
    bill_events: typing_extensions.Annotated[
        typing.Optional[BillEvents], FieldMetadata(alias="billEvents"), pydantic.Field(alias="billEvents")
    ] = None
    bill_approvals: typing_extensions.Annotated[
        typing.Optional[BillApprovals], FieldMetadata(alias="billApprovals"), pydantic.Field(alias="billApprovals")
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname"), pydantic.Field(alias="PaypointLegalname")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[
        typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname"), pydantic.Field(alias="PaypointDbaname")
    ] = None
    parent_org_id: typing_extensions.Annotated[
        typing.Optional[OrgParentId], FieldMetadata(alias="ParentOrgId"), pydantic.Field(alias="ParentOrgId")
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName"), pydantic.Field(alias="ParentOrgName")
    ] = None
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield],
        FieldMetadata(alias="PaypointEntryname"),
        pydantic.Field(alias="PaypointEntryname"),
    ] = None
    paylink_id: typing_extensions.Annotated[
        typing.Optional[PaylinkId], FieldMetadata(alias="paylinkId"), pydantic.Field(alias="paylinkId")
    ] = None
    documents_ref: typing_extensions.Annotated[
        typing.Optional[DocumentsRef],
        FieldMetadata(alias="DocumentsRef"),
        pydantic.Field(alias="DocumentsRef", description="Object with the attached documents."),
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None
    lot_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="LotNumber"),
        pydantic.Field(alias="LotNumber", description="Lot number of the bill."),
    ] = None
    entity_id: typing_extensions.Annotated[
        typing.Optional[EntityId], FieldMetadata(alias="EntityID"), pydantic.Field(alias="EntityID")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .authcode import Authcode
from .avs_response_text import AvsResponseText
from .customeridtrans import Customeridtrans
from .cvv_response_text import CvvResponseText
from .method_reference_id import MethodReferenceId
from .referenceidtrans import Referenceidtrans
from .result_code import ResultCode
from .resulttext import Resulttext


class PayabliApiResponse0ResponseData(UniversalBaseModel):
    auth_code: typing_extensions.Annotated[
        typing.Optional[Authcode], FieldMetadata(alias="AuthCode"), pydantic.Field(alias="AuthCode")
    ] = None
    avs_response_text: typing_extensions.Annotated[
        typing.Optional[AvsResponseText],
        FieldMetadata(alias="avsResponseText"),
        pydantic.Field(alias="avsResponseText"),
    ] = None
    customer_id: typing_extensions.Annotated[
        typing.Optional[Customeridtrans], FieldMetadata(alias="CustomerId"), pydantic.Field(alias="CustomerId")
    ] = None
    cvv_response_text: typing_extensions.Annotated[
        typing.Optional[CvvResponseText],
        FieldMetadata(alias="cvvResponseText"),
        pydantic.Field(alias="cvvResponseText"),
    ] = None
    method_reference_id: typing_extensions.Annotated[
        typing.Optional[MethodReferenceId],
        FieldMetadata(alias="methodReferenceId"),
        pydantic.Field(alias="methodReferenceId"),
    ] = None
    reference_id: typing_extensions.Annotated[
        typing.Optional[Referenceidtrans], FieldMetadata(alias="ReferenceId"), pydantic.Field(alias="ReferenceId")
    ] = None
    result_code: typing_extensions.Annotated[
        typing.Optional[ResultCode], FieldMetadata(alias="ResultCode"), pydantic.Field(alias="ResultCode")
    ] = None
    result_text: typing_extensions.Annotated[
        typing.Optional[Resulttext], FieldMetadata(alias="ResultText"), pydantic.Field(alias="ResultText")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .accept_oauth import AcceptOauth
from .accept_register import AcceptRegister
from .entry_attributes import EntryAttributes
from .last_modified import LastModified
from .org_parent_name import OrgParentName
from .reference_name import ReferenceName
from .reference_template_id import ReferenceTemplateId
from .template_code import TemplateCode
from .template_name import TemplateName


class QueryBoardingLinksResponseRecordsItem(UniversalBaseModel):
    accept_oauth: typing_extensions.Annotated[
        typing.Optional[AcceptOauth], FieldMetadata(alias="AcceptOauth"), pydantic.Field(alias="AcceptOauth")
    ] = None
    accept_register: typing_extensions.Annotated[
        typing.Optional[AcceptRegister], FieldMetadata(alias="AcceptRegister"), pydantic.Field(alias="AcceptRegister")
    ] = None
    entry_attributes: typing_extensions.Annotated[
        typing.Optional[EntryAttributes],
        FieldMetadata(alias="EntryAttributes"),
        pydantic.Field(alias="EntryAttributes"),
    ] = None
    id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="Id"), pydantic.Field(alias="Id", description="The record ID.")
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[LastModified], FieldMetadata(alias="LastUpdated"), pydantic.Field(alias="LastUpdated")
    ] = None
    org_parent_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="OrgParentName"), pydantic.Field(alias="OrgParentName")
    ] = None
    reference_name: typing_extensions.Annotated[
        typing.Optional[ReferenceName], FieldMetadata(alias="ReferenceName"), pydantic.Field(alias="ReferenceName")
    ] = None
    reference_template_id: typing_extensions.Annotated[
        typing.Optional[ReferenceTemplateId],
        FieldMetadata(alias="ReferenceTemplateId"),
        pydantic.Field(alias="ReferenceTemplateId"),
    ] = None
    template_code: typing_extensions.Annotated[
        typing.Optional[TemplateCode], FieldMetadata(alias="TemplateCode"), pydantic.Field(alias="TemplateCode")
    ] = None
    template_name: typing_extensions.Annotated[
        typing.Optional[TemplateName], FieldMetadata(alias="TemplateName"), pydantic.Field(alias="TemplateName")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .accounting_field import AccountingField
from .bill_item import BillItem
from .bill_query_record_2_bill_approvals_item import BillQueryRecord2BillApprovalsItem
from .bill_query_record_2_payment_method import BillQueryRecord2PaymentMethod
from .billstatus import Billstatus
from .created_at import CreatedAt
from .datenullable import Datenullable
from .datetime_nullable import DatetimeNullable
from .dbaname import Dbaname
from .external_paypoint_id import ExternalPaypointId
from .frequency import Frequency
from .general_events import GeneralEvents
from .legalname import Legalname
from .org_parent_name import OrgParentName
from .payment_id_string import PaymentIdString
from .terms import Terms
from .transaction_out_query_record import TransactionOutQueryRecord
from .vendor_out_data import VendorOutData


class BillQueryRecord2(UniversalBaseModel):
    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField],
        FieldMetadata(alias="AccountingField1"),
        pydantic.Field(alias="AccountingField1"),
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField],
        FieldMetadata(alias="AccountingField2"),
        pydantic.Field(alias="AccountingField2"),
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="AdditionalData"),
        pydantic.Field(alias="AdditionalData", description="Additional data associated with the bill."),
    ] = None
    batch_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BatchNumber"),
        pydantic.Field(alias="BatchNumber", description="Batch number associated with the bill."),
    ] = None
    bill_approvals: typing_extensions.Annotated[
        typing.Optional[typing.List[BillQueryRecord2BillApprovalsItem]],
        FieldMetadata(alias="billApprovals"),
        pydantic.Field(alias="billApprovals"),
    ] = None
    bill_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="BillDate"),
        pydantic.Field(
            alias="BillDate", description="Bill creation date in one of the accepted formats: YYYY-MM-DD, MM/DD/YYYY."
        ),
    ] = None
    bill_events: typing_extensions.Annotated[
        typing.Optional[typing.List[GeneralEvents]],
        FieldMetadata(alias="billEvents"),
        pydantic.Field(alias="billEvents", description="Events associated with the bill."),
    ] = None
    bill_items: typing_extensions.Annotated[
        typing.Optional[typing.List[BillItem]],
        FieldMetadata(alias="BillItems"),
        pydantic.Field(alias="BillItems", description="Array of items included in the bill."),
    ] = None
    bill_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillNumber"),
        pydantic.Field(alias="BillNumber", description="Bill number."),
    ] = None
    comments: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Comments"),
        pydantic.Field(alias="Comments", description="Additional comments on the bill."),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[CreatedAt],
        FieldMetadata(alias="CreatedAt"),
        pydantic.Field(alias="CreatedAt", description="Timestamp of when bill was created, in UTC."),
    ] = None
    discount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="Discount"),
        pydantic.Field(alias="Discount", description="Discount amount applied to the bill."),
    ] = None
    documents_ref: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="DocumentsRef"),
        pydantic.Field(alias="DocumentsRef", description="Reference to documents associated with the bill."),
    ] = None
    due_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="DueDate"),
        pydantic.Field(
            alias="DueDate", description="Bill due date in one of the accepted formats: YYYY-MM-DD, MM/DD/YYYY."
        ),
    ] = None
    end_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="EndDate"),
        pydantic.Field(alias="EndDate", description="End date for the bill."),
    ] = None
    entity_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="EntityID"),
        pydantic.Field(alias="EntityID", description="Entity identifier associated with the bill."),
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None
    frequency: typing_extensions.Annotated[
        typing.Optional[Frequency],
        FieldMetadata(alias="Frequency"),
        pydantic.Field(alias="Frequency", description="Frequency for scheduled bills. Applied only in `Mode` = 1."),
    ] = None
    id_bill: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="IdBill"),
        pydantic.Field(alias="IdBill", description="Identifier of the bill."),
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[DatetimeNullable],
        FieldMetadata(alias="LastUpdated"),
        pydantic.Field(alias="LastUpdated", description="Timestamp of when bill was last updated, in UTC."),
    ] = None
    lot_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="LotNumber"),
        pydantic.Field(alias="LotNumber", description="Lot number associated with the bill."),
    ] = None
    mode: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="Mode"),
        pydantic.Field(
            alias="Mode", description="Bill mode: value `0` for single/one-time bills, `1` for scheduled bills."
        ),
    ] = None
    net_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="NetAmount"),
        pydantic.Field(alias="NetAmount", description="Net amount of the bill."),
    ] = None
    parent_org_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ParentOrgId"),
        pydantic.Field(alias="ParentOrgId", description="Parent organization identifier."),
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName"), pydantic.Field(alias="ParentOrgName")
    ] = None
    payment_id: typing_extensions.Annotated[
        typing.Optional[PaymentIdString], FieldMetadata(alias="PaymentId"), pydantic.Field(alias="PaymentId")
    ] = None
    payment_method: typing_extensions.Annotated[
        typing.Optional[BillQueryRecord2PaymentMethod],
        FieldMetadata(alias="PaymentMethod"),
        pydantic.Field(alias="PaymentMethod", description="Preferred payment method used."),
    ] = None
    paylink_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="paylinkId"),
        pydantic.Field(alias="paylinkId", description="Paylink identifier associated with the bill."),
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[
        typing.Optional[Dbaname],
        FieldMetadata(alias="PaypointDbaname"),
        pydantic.Field(alias="PaypointDbaname", description="The paypoint's DBA name."),
    ] = None
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaypointEntryname"),
        pydantic.Field(alias="PaypointEntryname", description="Entry name of the paypoint."),
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname],
        FieldMetadata(alias="PaypointLegalname"),
        pydantic.Field(alias="PaypointLegalname", description="The paypoint's legal name."),
    ] = None
    source: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Source"),
        pydantic.Field(alias="Source", description="Source of the bill."),
    ] = None
    status: typing_extensions.Annotated[
        typing.Optional[Billstatus], FieldMetadata(alias="Status"), pydantic.Field(alias="Status")
    ] = None
    terms: typing_extensions.Annotated[
        typing.Optional[Terms],
        FieldMetadata(alias="Terms"),
        pydantic.Field(
            alias="Terms",
            description="The payment terms for invoice. If no terms were defined initially, then response data for this field will default to `N30`.",
        ),
    ] = None
    total_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="TotalAmount"),
        pydantic.Field(alias="TotalAmount", description="Total amount of the bill including taxes and fees."),
    ] = None
    transaction: typing_extensions.Annotated[
        typing.Optional[TransactionOutQueryRecord],
        FieldMetadata(alias="Transaction"),
        pydantic.Field(alias="Transaction", description="MoneyOut transaction associated to the bill."),
    ] = None
    vendor: typing_extensions.Annotated[
        typing.Optional[VendorOutData], FieldMetadata(alias="Vendor"), pydantic.Field(alias="Vendor")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.bill_data import BillData
from ...types.bill_options import BillOptions
from ...types.payor_data_request import PayorDataRequest


class InvoiceDataRequest(UniversalBaseModel):
    customer_data: typing_extensions.Annotated[
        typing.Optional[PayorDataRequest],
        FieldMetadata(alias="customerData"),
        pydantic.Field(
            alias="customerData",
            description="Object describing the customer/payor. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings. ",
        ),
    ] = None
    invoice_data: typing_extensions.Annotated[
        typing.Optional[BillData],
        FieldMetadata(alias="invoiceData"),
        pydantic.Field(alias="invoiceData", description="Object describing the invoice. Required for POST requests."),
    ] = None
    scheduled_options: typing_extensions.Annotated[
        typing.Optional[BillOptions],
        FieldMetadata(alias="scheduledOptions"),
        pydantic.Field(alias="scheduledOptions", description="Object with options for scheduled invoices."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TransactionDetailEvent(UniversalBaseModel):
    """
    Event associated with transaction processing
    """

    trans_event: typing_extensions.Annotated[str, FieldMetadata(alias="transEvent"), pydantic.Field(alias="transEvent")]
    event_data: typing_extensions.Annotated[str, FieldMetadata(alias="eventData"), pydantic.Field(alias="eventData")]
    event_time: typing_extensions.Annotated[str, FieldMetadata(alias="eventTime"), pydantic.Field(alias="eventTime")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .page_identifier import PageIdentifier
from .pagesize import Pagesize


class QueryPayoutTransactionSummary(UniversalBaseModel):
    page_identifier: typing_extensions.Annotated[
        typing.Optional[PageIdentifier], FieldMetadata(alias="pageIdentifier"), pydantic.Field(alias="pageIdentifier")
    ] = None
    page_size: typing_extensions.Annotated[
        typing.Optional[Pagesize], FieldMetadata(alias="pageSize"), pydantic.Field(alias="pageSize")
    ] = None
    total_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalAmount"), pydantic.Field(alias="totalAmount")
    ] = None
    total_authorized: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalAuthorized"), pydantic.Field(alias="totalAuthorized")
    ] = None
    total_authorized_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="totalAuthorizedAmount"),
        pydantic.Field(alias="totalAuthorizedAmount"),
    ] = None
    total_canceled: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalCanceled"), pydantic.Field(alias="totalCanceled")
    ] = None
    total_canceled_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalCanceledAmount"), pydantic.Field(alias="totalCanceledAmount")
    ] = None
    total_captured: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalCaptured"), pydantic.Field(alias="totalCaptured")
    ] = None
    total_captured_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalCapturedAmount"), pydantic.Field(alias="totalCapturedAmount")
    ] = None
    total_net_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalNetAmount"), pydantic.Field(alias="totalNetAmount")
    ] = None
    total_open: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalOpen"), pydantic.Field(alias="totalOpen")
    ] = None
    total_open_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalOpenAmount"), pydantic.Field(alias="totalOpenAmount")
    ] = None
    total_pages: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalPages"), pydantic.Field(alias="totalPages")
    ] = None
    total_paid: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalPaid"), pydantic.Field(alias="totalPaid")
    ] = None
    total_paid_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalPaidAmount"), pydantic.Field(alias="totalPaidAmount")
    ] = None
    total_on_hold: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalOnHold"),
        pydantic.Field(alias="totalOnHold", description="Total number of transactions that are currently on hold."),
    ] = None
    total_on_hold_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="totalOnHoldAmount"),
        pydantic.Field(
            alias="totalOnHoldAmount", description="Total amount of transactions that are currently on hold."
        ),
    ] = None
    total_processing: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalProcessing"), pydantic.Field(alias="totalProcessing")
    ] = None
    total_processing_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="totalProcessingAmount"),
        pydantic.Field(alias="totalProcessingAmount"),
    ] = None
    total_records: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="totalRecords"), pydantic.Field(alias="totalRecords")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.accountexp import Accountexp
from ...types.accountid import Accountid
from ...types.accounttype import Accounttype
from ...types.bin_data import BinData
from ...types.holdername import Holdername
from ...types.initiator import Initiator
from ...types.maskedaccount import Maskedaccount
from ...types.orderdescription import Orderdescription
from ...types.sequence import Sequence
from ...types.signaturedata import Signaturedata
from ...types.stored_method_usage_type import StoredMethodUsageType
from ...types.storedmethodid import Storedmethodid
from .transaction_detail_payment_details import TransactionDetailPaymentDetails


class TransactionDetailPaymentData(UniversalBaseModel):
    """
    Payment method and transaction details
    """

    masked_account: typing_extensions.Annotated[
        Maskedaccount, FieldMetadata(alias="maskedAccount"), pydantic.Field(alias="maskedAccount")
    ]
    account_type: typing_extensions.Annotated[
        Accounttype, FieldMetadata(alias="accountType"), pydantic.Field(alias="accountType")
    ]
    account_exp: typing_extensions.Annotated[
        typing.Optional[Accountexp], FieldMetadata(alias="accountExp"), pydantic.Field(alias="accountExp")
    ] = None
    holder_name: typing_extensions.Annotated[
        Holdername, FieldMetadata(alias="holderName"), pydantic.Field(alias="holderName")
    ]
    stored_id: typing_extensions.Annotated[
        typing.Optional[Storedmethodid], FieldMetadata(alias="storedId"), pydantic.Field(alias="storedId")
    ] = None
    initiator: typing.Optional[Initiator] = None
    stored_method_usage_type: typing_extensions.Annotated[
        typing.Optional[StoredMethodUsageType],
        FieldMetadata(alias="storedMethodUsageType"),
        pydantic.Field(alias="storedMethodUsageType"),
    ] = None
    sequence: typing.Optional[Sequence] = None
    order_description: typing_extensions.Annotated[
        Orderdescription, FieldMetadata(alias="orderDescription"), pydantic.Field(alias="orderDescription")
    ]
    account_id: typing_extensions.Annotated[
        typing.Optional[Accountid], FieldMetadata(alias="accountId"), pydantic.Field(alias="accountId")
    ] = None
    signature_data: typing_extensions.Annotated[
        typing.Optional[Signaturedata], FieldMetadata(alias="signatureData"), pydantic.Field(alias="signatureData")
    ] = None
    bin_data: typing_extensions.Annotated[
        typing.Optional[BinData], FieldMetadata(alias="binData"), pydantic.Field(alias="binData")
    ] = None
    payment_details: typing_extensions.Annotated[
        TransactionDetailPaymentDetails, FieldMetadata(alias="paymentDetails"), pydantic.Field(alias="paymentDetails")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...types.customer_summary_record import CustomerSummaryRecord
from ...types.dbaname import Dbaname
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.legalname import Legalname
from ...types.method_query_records import MethodQueryRecords
from ...types.org_parent_name import OrgParentName
from ...types.page_identifier import PageIdentifier
from ...types.payor_data_request import PayorDataRequest
from ...types.subscription_query_records import SubscriptionQueryRecords


class GetMethodResponseResponseDataCustomersItem(PayorDataRequest):
    balance: typing.Optional[float] = pydantic.Field(default=None)
    """
    Customer's current balance
    """

    created: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation timestamp
    """

    customer_consent: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Any]],
        FieldMetadata(alias="customerConsent"),
        pydantic.Field(alias="customerConsent", description="Customer consent information"),
    ] = None
    customer_status: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="customerStatus"),
        pydantic.Field(alias="customerStatus", description="Status code for the customer"),
    ] = None
    customer_summary: typing_extensions.Annotated[
        typing.Optional[CustomerSummaryRecord],
        FieldMetadata(alias="customerSummary"),
        pydantic.Field(alias="customerSummary"),
    ] = None
    customer_username: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="customerUsername"),
        pydantic.Field(alias="customerUsername", description="Username of the customer"),
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastUpdated"),
        pydantic.Field(alias="lastUpdated", description="Last update timestamp"),
    ] = None
    mfa: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Multi-factor authentication status
    """

    mfa_mode: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="mfaMode"),
        pydantic.Field(alias="mfaMode", description="MFA mode setting"),
    ] = None
    pageindentifier: typing.Optional[PageIdentifier] = None
    parent_org_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="parentOrgId"),
        pydantic.Field(alias="parentOrgId", description="Parent organization ID"),
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="parentOrgName"), pydantic.Field(alias="parentOrgName")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[
        typing.Optional[Dbaname], FieldMetadata(alias="paypointDbaname"), pydantic.Field(alias="paypointDbaname")
    ] = None
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="paypointEntryname"),
        pydantic.Field(alias="paypointEntryname", description="The paypoint entryname the customer is associated with"),
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="paypointLegalname"), pydantic.Field(alias="paypointLegalname")
    ] = None
    sn_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Any]],
        FieldMetadata(alias="snData"),
        pydantic.Field(alias="snData", description="Social network data"),
    ] = None
    sn_identifier: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snIdentifier"),
        pydantic.Field(alias="snIdentifier", description="Social network identifier"),
    ] = None
    sn_provider: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snProvider"),
        pydantic.Field(alias="snProvider", description="Social network provider"),
    ] = None
    stored_methods: typing_extensions.Annotated[
        typing.Optional[typing.List[MethodQueryRecords]],
        FieldMetadata(alias="storedMethods"),
        pydantic.Field(alias="storedMethods", description="List of payment methods associated to the customer"),
    ] = None
    subscriptions: typing.Optional[typing.List[SubscriptionQueryRecords]] = pydantic.Field(default=None)
    """
    List of subscriptions associated to the customer
    """

    time_zone: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="timeZone"),
        pydantic.Field(alias="timeZone", description="Customer's timezone"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .frequency import Frequency


class ScheduleDetail(UniversalBaseModel):
    end_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endDate"),
        pydantic.Field(
            alias="endDate",
            description="Subscription end date in any of the accepted formats: YYYY-MM-DD, MM/DD/YYYY or the value `untilcancelled` to indicate a scheduled payment with infinite cycle.",
        ),
    ] = None
    frequency: typing.Optional[Frequency] = pydantic.Field(default=None)
    """
    Frequency of the subscription.
    """

    plan_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="planId"),
        pydantic.Field(
            alias="planId",
            description="This field is for future development, leave null. Identifier of subscription plan applied in the scheduled payment/subscription.",
        ),
    ] = None
    start_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="startDate"),
        pydantic.Field(
            alias="startDate",
            description="Subscription start date in any of the accepted formats: YYYY-MM-DD, MM/DD/YYYY. This must be a future date.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.authcode import Authcode
from ...types.avs_response_text import AvsResponseText
from ...types.customer_id import CustomerId
from ...types.cvv_response_text import CvvResponseText
from ...types.expected_processing_date_time import ExpectedProcessingDateTime
from ...types.method_reference_id import MethodReferenceId
from ...types.referenceidtrans import Referenceidtrans
from ...types.result_code import ResultCode


class ResponseDataRefunds(UniversalBaseModel):
    auth_code: typing_extensions.Annotated[Authcode, FieldMetadata(alias="authCode"), pydantic.Field(alias="authCode")]
    expected_processing_date_time: typing_extensions.Annotated[
        typing.Optional[ExpectedProcessingDateTime],
        FieldMetadata(alias="expectedProcessingDateTime"),
        pydantic.Field(alias="expectedProcessingDateTime"),
    ] = None
    avs_response_text: typing_extensions.Annotated[
        typing.Optional[AvsResponseText],
        FieldMetadata(alias="avsResponseText"),
        pydantic.Field(alias="avsResponseText", description="This field isn't applicable to refund operations."),
    ] = None
    customer_id: typing_extensions.Annotated[
        typing.Optional[CustomerId], FieldMetadata(alias="customerId"), pydantic.Field(alias="customerId")
    ] = None
    cvv_response_text: typing_extensions.Annotated[
        typing.Optional[CvvResponseText],
        FieldMetadata(alias="cvvResponseText"),
        pydantic.Field(alias="cvvResponseText", description="This field isn't applicable to refund operations."),
    ] = None
    method_reference_id: typing_extensions.Annotated[
        typing.Optional[MethodReferenceId],
        FieldMetadata(alias="methodReferenceId"),
        pydantic.Field(alias="methodReferenceId", description="This field isn't applicable to refund operations."),
    ] = None
    reference_id: typing_extensions.Annotated[
        Referenceidtrans, FieldMetadata(alias="referenceId"), pydantic.Field(alias="referenceId")
    ]
    result_code: typing_extensions.Annotated[
        ResultCode, FieldMetadata(alias="resultCode"), pydantic.Field(alias="resultCode")
    ]
    result_text: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="resultText"),
        pydantic.Field(alias="resultText", description="Text description of the transaction result"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.api_error import ApiError
from ..types.v_2_declined_transaction_response_wrapper import V2DeclinedTransactionResponseWrapper


class DeclinedVoidResponseErrorV2(ApiError):
    def __init__(
        self, body: V2DeclinedTransactionResponseWrapper, headers: typing.Optional[typing.Dict[str, str]] = None
    ):
        super().__init__(status_code=402, headers=headers, body=body)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class StatBasicQueryRecord(UniversalBaseModel):
    stat_x: typing_extensions.Annotated[
        str, FieldMetadata(alias="statX"), pydantic.Field(alias="statX", description="Statistical grouping identifier")
    ]
    in_transactions: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="inTransactions"),
        pydantic.Field(alias="inTransactions", description="Number of incoming transactions"),
    ]
    in_transactions_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="inTransactionsVolume"),
        pydantic.Field(alias="inTransactionsVolume", description="Volume of incoming transactions"),
    ]
    in_wallet_transactions: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="inWalletTransactions"),
        pydantic.Field(alias="inWalletTransactions", description="Number of incoming wallet transactions"),
    ]
    in_wallet_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="inWalletVolume"),
        pydantic.Field(alias="inWalletVolume", description="Volume of incoming wallet transactions"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        V2BadRequestError,
        V2BadRequestErrorDetail,
        V2DeclinedTransactionResponseWrapper,
        V2InternalServerError,
        V2ResponseAction,
        V2ResponseCode,
        V2ResponseExplanation,
        V2ResponseReason,
        V2TransactionDetailResponseData,
        V2TransactionDetails,
        V2TransactionResponseWrapper,
    )
    from .errors import (
        BadRequestAuthResponseErrorV2,
        BadRequestCaptureResponseErrorV2,
        BadRequestRefundResponseErrorV2,
        BadRequestVoidResponseErrorV2,
        DeclinedAuthResponseErrorV2,
        DeclinedCaptureResponseErrorV2,
        DeclinedRefundResponseErrorV2,
        DeclinedVoidResponseErrorV2,
        InternalServerResponseErrorV2,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "BadRequestAuthResponseErrorV2": ".errors",
    "BadRequestCaptureResponseErrorV2": ".errors",
    "BadRequestRefundResponseErrorV2": ".errors",
    "BadRequestVoidResponseErrorV2": ".errors",
    "DeclinedAuthResponseErrorV2": ".errors",
    "DeclinedCaptureResponseErrorV2": ".errors",
    "DeclinedRefundResponseErrorV2": ".errors",
    "DeclinedVoidResponseErrorV2": ".errors",
    "InternalServerResponseErrorV2": ".errors",
    "V2BadRequestError": ".types",
    "V2BadRequestErrorDetail": ".types",
    "V2DeclinedTransactionResponseWrapper": ".types",
    "V2InternalServerError": ".types",
    "V2ResponseAction": ".types",
    "V2ResponseCode": ".types",
    "V2ResponseExplanation": ".types",
    "V2ResponseReason": ".types",
    "V2TransactionDetailResponseData": ".types",
    "V2TransactionDetails": ".types",
    "V2TransactionResponseWrapper": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadRequestAuthResponseErrorV2",
    "BadRequestCaptureResponseErrorV2",
    "BadRequestRefundResponseErrorV2",
    "BadRequestVoidResponseErrorV2",
    "DeclinedAuthResponseErrorV2",
    "DeclinedCaptureResponseErrorV2",
    "DeclinedRefundResponseErrorV2",
    "DeclinedVoidResponseErrorV2",
    "InternalServerResponseErrorV2",
    "V2BadRequestError",
    "V2BadRequestErrorDetail",
    "V2DeclinedTransactionResponseWrapper",
    "V2InternalServerError",
    "V2ResponseAction",
    "V2ResponseCode",
    "V2ResponseExplanation",
    "V2ResponseReason",
    "V2TransactionDetailResponseData",
    "V2TransactionDetails",
    "V2TransactionResponseWrapper",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class VendorDataRequest(UniversalBaseModel):
    """
    Object describing the vendor owner of payment method. Required when saving an ACH payment method on behalf of a vendor (for Pay Out transactions).
    """

    vendor_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="vendorId"),
        pydantic.Field(
            alias="vendorId",
            description="The unique numeric ID assigned to the vendor in Payabli. Either `vendorId` or `vendorNumber` is required.",
        ),
    ] = None
    vendor_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="vendorNumber"),
        pydantic.Field(
            alias="vendorNumber",
            description="Custom vendor number assigned by the business. Either `vendorId` or `vendorNumber` is required.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .query_transaction_events_event_data import QueryTransactionEventsEventData


class QueryTransactionEvents(UniversalBaseModel):
    event_data: typing_extensions.Annotated[
        typing.Optional[QueryTransactionEventsEventData],
        FieldMetadata(alias="EventData"),
        pydantic.Field(
            alias="EventData",
            description="Any data associated to the event received from processor. Contents vary by event type.",
        ),
    ] = None
    event_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="EventTime"),
        pydantic.Field(alias="EventTime", description="Date and time of event."),
    ] = None
    trans_event: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="TransEvent"),
        pydantic.Field(
            alias="TransEvent",
            description="Event descriptor. See [TransEvent Reference](/guides/pay-in-transevents-reference) for more details.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class NotificationLogSearchRequest(UniversalBaseModel):
    start_date: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="startDate"),
        pydantic.Field(alias="startDate", description="The start date for the search."),
    ]
    end_date: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="endDate"),
        pydantic.Field(alias="endDate", description="The end date for the search."),
    ]
    notification_event: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="notificationEvent"),
        pydantic.Field(alias="notificationEvent", description="The type of notification event to filter by."),
    ] = None
    succeeded: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the notification was successful.
    """

    org_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="orgId"),
        pydantic.Field(alias="orgId", description="The ID of the organization to filter by."),
    ] = None
    paypoint_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="paypointId"),
        pydantic.Field(alias="paypointId", description="The ID of the paypoint to filter by."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class QueryBatchesTransfer(UniversalBaseModel):
    """
    Transfer details within a batch response.
    """

    transfer_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="TransferId"),
        pydantic.Field(alias="TransferId", description="The transfer ID."),
    ] = None
    transfer_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="TransferDate"),
        pydantic.Field(alias="TransferDate", description="The transfer date."),
    ] = None
    processor: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Processor"),
        pydantic.Field(alias="Processor", description="The processor used for the transfer."),
    ] = None
    transfer_status: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="TransferStatus"),
        pydantic.Field(alias="TransferStatus", description="The transfer status."),
    ] = None
    gross_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="GrossAmount"),
        pydantic.Field(alias="GrossAmount", description="The gross amount of the transfer."),
    ] = None
    charge_back_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="ChargeBackAmount"),
        pydantic.Field(alias="ChargeBackAmount", description="The chargeback amount."),
    ] = None
    returned_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="ReturnedAmount"),
        pydantic.Field(alias="ReturnedAmount", description="The returned amount."),
    ] = None
    refund_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="RefundAmount"),
        pydantic.Field(alias="RefundAmount", description="The refund amount."),
    ] = None
    hold_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="HoldAmount"),
        pydantic.Field(alias="HoldAmount", description="The amount being held."),
    ] = None
    released_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="ReleasedAmount"),
        pydantic.Field(alias="ReleasedAmount", description="The amount that has been released."),
    ] = None
    billing_fees_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="BillingFeesAmount"),
        pydantic.Field(alias="BillingFeesAmount", description="The billing fees amount."),
    ] = None
    third_party_paid_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="ThirdPartyPaidAmount"),
        pydantic.Field(alias="ThirdPartyPaidAmount", description="The third party paid amount."),
    ] = None
    adjustments_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="AdjustmentsAmount"),
        pydantic.Field(alias="AdjustmentsAmount", description="The adjustments amount."),
    ] = None
    net_funded_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="NetFundedAmount"),
        pydantic.Field(alias="NetFundedAmount", description="The net funded amount."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .query_entrypoint_response_records_item import QueryEntrypointResponseRecordsItem
from .query_summary import QuerySummary


class QueryEntrypointResponse(UniversalBaseModel):
    records: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryEntrypointResponseRecordsItem]],
        FieldMetadata(alias="Records"),
        pydantic.Field(alias="Records"),
    ] = None
    summary: typing_extensions.Annotated[
        typing.Optional[QuerySummary], FieldMetadata(alias="Summary"), pydantic.Field(alias="Summary")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .bill_details_response import BillDetailsResponse
from .comments import Comments
from .created_at import CreatedAt
from .dbaname import Dbaname
from .entry import Entry
from .external_paypoint_id import ExternalPaypointId
from .fee_amount import FeeAmount
from .file_content import FileContent
from .gatewayfield import Gatewayfield
from .has_vcard_transactions import HasVcardTransactions
from .is_same_day_ach import IsSameDayAch
from .last_modified import LastModified
from .legalname import Legalname
from .netamountnullable import Netamountnullable
from .org_parent_id import OrgParentId
from .org_parent_name import OrgParentName
from .payment_id_string import PaymentIdString
from .query_payment_data import QueryPaymentData
from .query_transaction_events import QueryTransactionEvents
from .risk_action import RiskAction
from .risk_action_code import RiskActionCode
from .risk_flagged import RiskFlagged
from .risk_flagged_on import RiskFlaggedOn
from .risk_reason import RiskReason
from .risk_status import RiskStatus
from .schedule_id import ScheduleId
from .settlement_status import SettlementStatus
from .source import Source
from .vendor_query_record import VendorQueryRecord


class BillDetailResponse(UniversalBaseModel):
    bills: typing_extensions.Annotated[
        typing.Optional[typing.List[BillDetailsResponse]],
        FieldMetadata(alias="Bills"),
        pydantic.Field(alias="Bills", description="Events associated to this transaction."),
    ] = None
    check_data: typing_extensions.Annotated[
        typing.Optional[FileContent],
        FieldMetadata(alias="CheckData"),
        pydantic.Field(alias="CheckData", description="Object referencing to paper check image."),
    ] = None
    check_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="CheckNumber"),
        pydantic.Field(alias="CheckNumber", description="Paper check number related to payout transaction."),
    ] = None
    comments: typing_extensions.Annotated[
        typing.Optional[Comments],
        FieldMetadata(alias="Comments"),
        pydantic.Field(alias="Comments", description="Any comment or description for payout transaction."),
    ] = None
    created_date: typing_extensions.Annotated[
        typing.Optional[CreatedAt],
        FieldMetadata(alias="CreatedDate"),
        pydantic.Field(alias="CreatedDate", description="Timestamp when the payment was created, in UTC."),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt"), pydantic.Field(alias="CreatedAt")
    ] = None
    events: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryTransactionEvents]],
        FieldMetadata(alias="Events"),
        pydantic.Field(alias="Events", description="Events associated to this transaction."),
    ] = None
    fee_amount: typing_extensions.Annotated[
        typing.Optional[FeeAmount], FieldMetadata(alias="FeeAmount"), pydantic.Field(alias="FeeAmount")
    ] = None
    gateway: typing_extensions.Annotated[
        typing.Optional[Gatewayfield], FieldMetadata(alias="Gateway"), pydantic.Field(alias="Gateway")
    ] = None
    id_out: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="IdOut"),
        pydantic.Field(alias="IdOut", description="Identifier of payout transaction."),
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[LastModified],
        FieldMetadata(alias="LastUpdated"),
        pydantic.Field(alias="LastUpdated", description="Timestamp when payment record was updated, in UTC."),
    ] = None
    net_amount: typing_extensions.Annotated[
        typing.Optional[Netamountnullable], FieldMetadata(alias="NetAmount"), pydantic.Field(alias="NetAmount")
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName"), pydantic.Field(alias="ParentOrgName")
    ] = None
    parent_org_id: typing_extensions.Annotated[
        typing.Optional[OrgParentId], FieldMetadata(alias="ParentOrgId"), pydantic.Field(alias="ParentOrgId")
    ] = None
    payment_data: typing_extensions.Annotated[
        typing.Optional[QueryPaymentData], FieldMetadata(alias="PaymentData"), pydantic.Field(alias="PaymentData")
    ] = None
    payment_group: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentGroup"),
        pydantic.Field(
            alias="PaymentGroup", description="Unique identifier for group or batch containing the transaction."
        ),
    ] = None
    payment_id: typing_extensions.Annotated[
        typing.Optional[PaymentIdString], FieldMetadata(alias="PaymentId"), pydantic.Field(alias="PaymentId")
    ] = None
    payment_method: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentMethod"),
        pydantic.Field(alias="PaymentMethod", description="Method of payment applied to the transaction."),
    ] = None
    payment_status: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PaymentStatus"),
        pydantic.Field(alias="PaymentStatus", description="Status of payout transaction."),
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[
        typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname"), pydantic.Field(alias="PaypointDbaname")
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname],
        FieldMetadata(alias="PaypointLegalname"),
        pydantic.Field(alias="PaypointLegalname", description="Paypoint legal name."),
    ] = None
    source: typing_extensions.Annotated[
        typing.Optional[Source], FieldMetadata(alias="Source"), pydantic.Field(alias="Source")
    ] = None
    status: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="Status"),
        pydantic.Field(
            alias="Status",
            description="Internal status of transaction.\n\nPayout statuses, also known as money out transaction statuses, appear in PartnerHub and PayHub, and the API, and describe where a payout transaction is in its lifecycle.\n\n| Status | Key | Description | Events |\n|--------|-----|-------------|---------|\n| **Authorized** | 11 | A payout is authorized. These are queued payouts, and nothing happens with them until they're captured. | Authorized |\n| **Captured** | 1 | A payout is captured and is now part of the batch for payout. | Captured |\n| **Canceled** | 0 | An authorized payout has been canceled. A captured payout can be canceled before batch close at 5 PM ET. | Cancelled |\n| **Processing** | 2 | A payout is being processed. | Waiting funds, Funded, Pending (payment type is pending), Generating check |\n| **Processed** | 3 | A payment method is defined for the vendor, and the payout has been sent to the recipient. | Open (vCard issued, ACH sent, check generated but not yet cashed), Processed (Payment Type is no longer pending), Reissued, Returned, Errored |\n| **OnHold** | 4 | A payout has been placed on hold and requires review before proceeding. | OnHold |\n| **Paid** | 5 | A payout has been paid and the recipient has redeemed the funds. | Paid (check cleared, vCard used, ACH settled) |",
        ),
    ] = None
    status_text: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="StatusText"),
        pydantic.Field(alias="StatusText", description="Status of payout transaction."),
    ] = None
    total_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="TotalAmount"),
        pydantic.Field(
            alias="TotalAmount", description="Transaction total amount (including service fee or sub-charge)."
        ),
    ] = None
    vendor: typing_extensions.Annotated[
        typing.Optional[VendorQueryRecord],
        FieldMetadata(alias="Vendor"),
        pydantic.Field(alias="Vendor", description="Vendor related to the payout transaction."),
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None
    entry_name: typing_extensions.Annotated[
        typing.Optional[Entry], FieldMetadata(alias="EntryName"), pydantic.Field(alias="EntryName")
    ] = None
    batch_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BatchId"),
        pydantic.Field(
            alias="BatchId",
            description="Identifier for the batch in which this transaction was processed. Used to track and reconcile batch-level operations.",
        ),
    ] = None
    has_vcard_transactions: typing_extensions.Annotated[
        typing.Optional[HasVcardTransactions],
        FieldMetadata(alias="HasVcardTransactions"),
        pydantic.Field(alias="HasVcardTransactions"),
    ] = None
    is_same_day_ach: typing_extensions.Annotated[
        typing.Optional[IsSameDayAch], FieldMetadata(alias="IsSameDayACH"), pydantic.Field(alias="IsSameDayACH")
    ] = None
    schedule_id: typing_extensions.Annotated[
        typing.Optional[ScheduleId], FieldMetadata(alias="ScheduleId"), pydantic.Field(alias="ScheduleId")
    ] = None
    settlement_status: typing_extensions.Annotated[
        typing.Optional[SettlementStatus],
        FieldMetadata(alias="SettlementStatus"),
        pydantic.Field(alias="SettlementStatus"),
    ] = None
    risk_flagged: typing_extensions.Annotated[
        typing.Optional[RiskFlagged], FieldMetadata(alias="RiskFlagged"), pydantic.Field(alias="RiskFlagged")
    ] = None
    risk_flagged_on: typing_extensions.Annotated[
        typing.Optional[RiskFlaggedOn], FieldMetadata(alias="RiskFlaggedOn"), pydantic.Field(alias="RiskFlaggedOn")
    ] = None
    risk_status: typing_extensions.Annotated[
        typing.Optional[RiskStatus], FieldMetadata(alias="RiskStatus"), pydantic.Field(alias="RiskStatus")
    ] = None
    risk_reason: typing_extensions.Annotated[
        typing.Optional[RiskReason], FieldMetadata(alias="RiskReason"), pydantic.Field(alias="RiskReason")
    ] = None
    risk_action: typing_extensions.Annotated[
        typing.Optional[RiskAction], FieldMetadata(alias="RiskAction"), pydantic.Field(alias="RiskAction")
    ] = None
    risk_action_code: typing_extensions.Annotated[
        typing.Optional[RiskActionCode], FieldMetadata(alias="RiskActionCode"), pydantic.Field(alias="RiskActionCode")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VendorResponseStoredMethod(UniversalBaseModel):
    """
    Stored payment method information
    """

    id_pmethod: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="IdPmethod"), pydantic.Field(alias="IdPmethod")
    ] = None
    method: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="Method"), pydantic.Field(alias="Method")
    ] = None
    descriptor: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="Descriptor"), pydantic.Field(alias="Descriptor")
    ] = None
    masked_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="MaskedAccount"), pydantic.Field(alias="MaskedAccount")
    ] = None
    exp_date: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="ExpDate"), pydantic.Field(alias="ExpDate")
    ] = None
    holder_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="HolderName"), pydantic.Field(alias="HolderName")
    ] = None
    ach_sec_code: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="AchSecCode"), pydantic.Field(alias="AchSecCode")
    ] = None
    ach_holder_type: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="AchHolderType"), pydantic.Field(alias="AchHolderType")
    ] = None
    is_validated_ach: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="IsValidatedACH"), pydantic.Field(alias="IsValidatedACH")
    ] = None
    bin: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BIN"), pydantic.Field(alias="BIN")] = (
        None
    )
    bin_data: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="binData"), pydantic.Field(alias="binData")
    ] = None
    aba: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ABA"), pydantic.Field(alias="ABA")] = (
        None
    )
    postal_code: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="PostalCode"), pydantic.Field(alias="PostalCode")
    ] = None
    method_type: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="MethodType"), pydantic.Field(alias="MethodType")
    ] = None
    last_updated: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="LastUpdated"), pydantic.Field(alias="LastUpdated")
    ] = None
    card_updated_on: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="CardUpdatedOn"), pydantic.Field(alias="CardUpdatedOn")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
